// Includes
#include "LCD.h"

// Function Implementations
void LCDInit( void )
{
	//Set the LCD port to output mode
	LCD_PORT = 0x00;
	LCD_TRIS = 0x00;

	//Wait for the LCD to settle
	Delay( 0x64 );

	// Set 8 bit mode, 2 line mode and 5x7 mode
	LCDCmd( 0x38 );

	// Set display on, cursor on and cursor blink on
	LCDCmd( 0x0F );

	// Set increment mode and entire shift off
	LCDCmd( 0x06 );

	// Clear display
	LCDClear();
}

void LCDCmd( unsigned int iCommand )
{
	int iShiftRegisterBitIndex = 7;

	while( iShiftRegisterBitIndex >= 0 )
	{
		if( ( iCommand >> iShiftRegisterBitIndex ) & 0x01 )
		{
			PIN_ON( LCD_PORT, LCD_REG_DATA );
		}
		else
		{
			PIN_OFF( LCD_PORT, LCD_REG_DATA );
		}

		LCDStrobeReg();

		iShiftRegisterBitIndex--;
	}

	PIN_OFF( LCD_PORT, LCD_RS );

	LCDStrobeE();

	Delay( 0x05 );
}

void LCDData( unsigned int iData )
{
	int iShiftRegisterBitIndex = 7;

	while( iShiftRegisterBitIndex >= 0 )
	{
		if( ( iData >> iShiftRegisterBitIndex ) & 0x01 )
		{
			PIN_ON( LCD_PORT, LCD_REG_DATA );
		}
		else
		{
			PIN_OFF( LCD_PORT, LCD_REG_DATA );
		}

		LCDStrobeReg();

		iShiftRegisterBitIndex--;
	}

	PIN_ON( LCD_PORT, LCD_RS );

	LCDStrobeE();

	Delay( 0x05 );
}

void LCDStrobeE( void )
{
	PIN_ON( LCD_PORT, LCD_E );
	Nop();
	PIN_OFF( LCD_PORT, LCD_E );
}

void LCDStrobeReg( void )
{
	PIN_ON( LCD_PORT, LCD_REG_CLOCK );
	Nop();
	PIN_OFF( LCD_PORT, LCD_REG_CLOCK );
}

void LCDClear( void )
{
	LCDCmd( 0x01 );
}

void LCDCGRAM( unsigned int iCharacter )
{
	iCharacter *= 0x08; // Offset

	iCharacter += 0x40; // Set CG RAM instruction

	LCDCmd( iCharacter );
}

void LCDLine( unsigned int iLine )
{
	iLine *= 0x40; // Offset

	if( iLine > 0x40 )
	{
		iLine -= 0x6C; // Move into second controller's address
	}

	iLine += 0x80; // Set DD RAM instruction

	LCDCmd( iLine );
}

void LCDGoto( unsigned int iLine, unsigned int iColumn )
{
	iLine *= 0x40; // Offset

	if( iLine > 0x40 )
	{
		iLine -= 0x6C; // Move into second controller's address
	}

	iLine += iColumn; // Offset by column number

	iLine += 0x80; // Set DD RAM instruction

	LCDCmd( iLine );
}

void LCDPrint( const rom char* pString )
{
	while( *pString )
	{
		LCDData( *pString );

		pString++;
	}
}

void LCDPrintSlow( const rom char* pString, unsigned int iDelay )
{
	while( *pString )
	{
		LCDData( *pString );
		Delay( iDelay );

		pString++;
	}
}

void LCDFloatNumber2DP( float fNumber )
{
	LCDIntNumber2DP( fNumber * 100.00 );
}
	
void LCDIntNumber2DP( unsigned int iNumber )
{	
	unsigned int iRawNumber = iNumber;
	unsigned int iDecNumber[4] = { 0x00, 0x00, 0x00, 0x00 };
	unsigned int iIndex = 0x03;
	unsigned int iFactors[4] = { 0x00, 0x0A, 0x64, 0x3E8 };
	unsigned int iFactor = 0x00;

	while( iIndex != 0x00 )
	{
		iFactor = iFactors[iIndex];

		CLEAR( STATUS, 4 );
		iRawNumber -= iFactor;
		while( IS_NOT_SET( STATUS, 4 ) )
		{
			iDecNumber[iIndex]++;
			iRawNumber -= iFactor;
		}
		iRawNumber += iFactor;

		iIndex--;
	}

	iDecNumber[iIndex] = iRawNumber;

	LCDData( iDecNumber[3] + 0x30 );
	LCDData( iDecNumber[2] + 0x30 );
	LCDData( '.' );
	LCDData( iDecNumber[1] + 0x30 );
	LCDData( iDecNumber[0] + 0x30 );
}
